#include "M5_BM8563.h"

M5_BM8563::M5_BM8563(uint8_t deviceAddress, TwoWire &i2cPort) {
  _deviceAddress = deviceAddress;
  _i2cPort = &i2cPort;
}

void M5_BM8563::begin(void) {
  _i2cPort->beginTransmission(_deviceAddress);
  _i2cPort->write(0x00);
  _i2cPort->write(0x00);
  _i2cPort->write(0x00);
  _i2cPort->endTransmission();
}

uint8_t M5_BM8563::bcd2ToByte(uint8_t value) {
  uint8_t tmp = 0;
  tmp = ((uint8_t)(value & (uint8_t)0xF0) >> (uint8_t)0x4) * 10;
  return (tmp + (value & (uint8_t)0x0F));
}

uint8_t M5_BM8563::byteToBcd2(uint8_t value) {
  uint8_t bcdhigh = 0;

  while (value >= 10) {
    bcdhigh++;
    value -= 10;
  }

  return ((uint8_t)(bcdhigh << 4) | value);
}



void M5_BM8563::getTime(int *hours,int *minutes,int *seconds) {
  uint8_t buf[3] = {0};

  _i2cPort->beginTransmission(_deviceAddress);
  _i2cPort->write(0x02);
  _i2cPort->endTransmission();
  _i2cPort->requestFrom(_deviceAddress, 3);

  while (_i2cPort->available()) {
    buf[0] = _i2cPort->read();
    buf[1] = _i2cPort->read();
    buf[2] = _i2cPort->read();
  }

    *seconds = bcd2ToByte(buf[0] & 0x7f);
    *minutes = bcd2ToByte(buf[1] & 0x7f);
    *hours   = bcd2ToByte(buf[2] & 0x3f);
  
}

void M5_BM8563::setTime(int *hours,int *minutes,int *seconds) {
  //if (hours == NULL) {
   // return;
  //}

  _i2cPort->beginTransmission(_deviceAddress);
  _i2cPort->write(0x02);
  _i2cPort->write(byteToBcd2(*seconds));
  _i2cPort->write(byteToBcd2(*minutes));
  _i2cPort->write(byteToBcd2(*hours));
  _i2cPort->endTransmission();
}

void M5_BM8563::getDate(int *weekDay,int *month,int *day,int *year) {
  uint8_t buf[4] = {0};

  _i2cPort->beginTransmission(_deviceAddress);
  _i2cPort->write(0x05);
  _i2cPort->endTransmission();
  _i2cPort->requestFrom(_deviceAddress, 4);

  while (_i2cPort->available()) {
    buf[0] = _i2cPort->read();
    buf[1] = _i2cPort->read();
    buf[2] = _i2cPort->read();
    buf[3] = _i2cPort->read();
  }

  *day    = bcd2ToByte(buf[0] & 0x3f);
  *weekDay = bcd2ToByte(buf[1] & 0x07);
  *month   = bcd2ToByte(buf[2] & 0x1f);

  if (buf[2] & 0x80) {
    *year = 1900 + bcd2ToByte(buf[3] & 0xff);
  } else {
    *year = 2000 + bcd2ToByte(buf[3] & 0xff);
  }
}

void M5_BM8563::setDate(int *weekDay,int *month,int *day,int *year) {
  //if (I2C_BM8563_DateStruct == NULL) {
  //  return;
  //}

  _i2cPort->beginTransmission(_deviceAddress);
  _i2cPort->write(0x05);
  _i2cPort->write(byteToBcd2(*day));
  _i2cPort->write(byteToBcd2(*weekDay));

  if (*year < 2000) {
    _i2cPort->write(byteToBcd2(*month) | 0x80);
    _i2cPort->write(byteToBcd2((uint8_t)(*year % 100)));
  } else {
    _i2cPort->write(byteToBcd2(*month) | 0x00);
    _i2cPort->write(byteToBcd2((uint8_t)(*year % 100)));
  }

  _i2cPort->endTransmission();
}
